unit Wytecu;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Dialogs, ShellAPI;

type
  TfrmWytec = class(TForm)
    BtnDebug: TButton;
    BtnClose: TButton;
    Bevel1: TBevel;
    edtDownload: TEdit;
    Label1: TLabel;
    BtnBrowse: TButton;
    BtnProgrammer: TButton;
    BtnSelectFile: TButton;
    OpenDialog1: TOpenDialog;
    procedure BtnBrowseClick(Sender: TObject);
    procedure BtnDebugClick(Sender: TObject);
    procedure BtnProgrammerClick(Sender: TObject);
    procedure BtnCloseClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure BtnSelectFileClick(Sender: TObject);
  private
    { Private declarations }
    procedure SetupWytecConfigFile;
    function GetWytecConfigFilename: string;
  public
    { Public declarations }
    WytecFilename: string;
  end;

var
  frmWytec: TfrmWytec;

implementation

uses frmMain;

{$R *.DFM}

procedure TfrmWytec.SetupWytecConfigFile;
var
  sl: TStringList;
  BuildDir, Filename, ComPort, ep2Path: string;
begin
  if not FileExists(WytecFilename) then
    Exit;

  BuildDir := ExtractFilePath(WytecFilename);
  Filename := ExtractFileName(WytecFilename);

  ep2Path := ExtractFilePath(mainform.WytecDebuggerPath);

  // Wytec tools only support COM1 and COM2
  if mainform.WytecCOMPort = 'COM 1' then
    ComPort := 'COM1'
  else if mainform.WytecCOMPort = 'COM 2' then
    ComPort := 'COM2';

  sl := TStringList.Create;
  try
    sl.Add(BuildDir);
    sl.Add(ChangeFileExt(Filename, ''));
    sl.Add(ComPort);
    sl.Add(mainform.WytecMPUtype);
    sl.Add(mainform.WytecBoard);
    sl.SaveToFile(ep2Path + 'ep2ide.txt');
  finally
    sl.Free;
  end;
end;

procedure TfrmWytec.BtnBrowseClick(Sender: TObject);
begin
  with OpenDialog1 do
  begin
    Filename := edtDownload.Text;
    if Execute then
    begin
      edtDownload.Text := Filename;
      SetupWytecConfigFile;
    end;
  end;
end;

procedure TfrmWytec.BtnDebugClick(Sender: TObject);
begin
  if not FileExists(mainform.WytecDebuggerPath) then
  begin
    MessageDlg('Can not find debugger file: ' + mainform.WytecDebuggerPath + '  Please check View..Options', mtWarning, [mbOK], 0);
    Exit;
  end;
  if not FileExists(edtDownload.Text) then
  begin
    MessageDlg('Can not find file: ' + edtDownload.Text, mtError, [mbOK], 0);
    Exit;
  end;
  WytecFilename := edtDownload.Text;
  SetupWytecConfigFile;

  mainform.afComPort1.Close;
  ShellExecute(Application.MainForm.Handle, 'open', PChar(mainform.WytecDebuggerPath), nil, nil, SW_SHOWNORMAL);
  // we have to leave the com port closed - wayne doesn't have ascii I/O
end;

procedure TfrmWytec.BtnProgrammerClick(Sender: TObject);
begin
  if not FileExists(mainform.WytecProgrammerPath) then
  begin
    MessageDlg('Can not find programmer file: ' + mainform.WytecProgrammerPath + '  Please check View..Options', mtWarning, [mbOK], 0);
    Exit;
  end;
  if not FileExists(edtDownload.Text) then
  begin
    MessageDlg('Can not find file: ' + edtDownload.Text, mtError, [mbOK], 0);
    Exit;
  end;
  WytecFilename := edtDownload.Text;
  SetupWytecConfigFile;

  mainform.afComPort1.Close;
  ShellExecute(Application.MainForm.Handle, 'open', PChar(mainform.WytecProgrammerPath), nil, nil, SW_SHOWNORMAL);
  // we have to leave the com port closed - wayne doesn't have ascii I/O
end;

procedure TfrmWytec.BtnCloseClick(Sender: TObject);
begin
  self.Hide;
end;

procedure TfrmWytec.FormActivate(Sender: TObject);
begin
  if WytecFilename > '' then
    edtDownload.Text := WytecFilename
  else
    edtDownload.Text := GetWytecConfigFilename;
end;

procedure TfrmWytec.BtnSelectFileClick(Sender: TObject);
begin
  if not FileExists(edtDownload.Text) then
  begin
    MessageDlg('Can not find file: ' + edtDownload.Text, mtError, [mbOK], 0);
    Exit;
  end;
  WytecFilename := edtDownload.Text;
  SetupWytecConfigFile;
end;

// We didn't get a filename because there's no asm file open.
// Get the filename form the Wytec Config file.

function TfrmWytec.GetWytecConfigFilename: string;
var
  sl: TStringList;
  ep2path, BuildDir, Filename, DownloadFile: string;
begin
  try
    ep2Path := ExtractFilePath(mainform.WytecDebuggerPath);
    sl := TStringList.Create;
    try
      sl.LoadFromFile(ep2path + 'ep2ide.txt');
      if sl.Count < 2 then
        exit;
      BuildDir := sl[0];
      Filename := sl[1];
      DownloadFile := IncludeTrailingBackslash(BuildDir) + Filename + '.s19';
      if FileExists(Downloadfile) then
        Result := DownloadFile;
    finally
      sl.Free;
    end;
  except
    Result := '';
  end;
end;

end.
