{  AsmIDE by Eric W. Engler
http://www.EricEngler.com

The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.
}

unit frmAbout;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, LblEffct, ShellAPI, registry;

type
  TAboutBox = class(TForm)
    Panel1: TPanel;
    Version: TLabel;
    OKButton: TButton;
    ProgramIcon: TImage;
    Memo1: TMemo;
    lblHomePage: TLabel;
    Label4: TLabel;
    Label1: TLabel;
    lblFramework: TLabel;
    Label2: TLabel;
    lblWindows: TLabel;
    Label3: TLabel;
    lblService: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure lblHomePageClick(Sender: TObject);
  private
    LabelEffect1: TLabelEffect;
    function FrameworkVersion: string;
    function WindowsVersion: string;
    function ServicePackLevel: string;

  public
    { Public declarations }
  end;

var
  AboutBox: TAboutBox;

implementation

{$R *.DFM}

// We can have several versions of the Framework installed on the same computer,
// so this routine tries to identify them all.

function TAboutBox.FrameworkVersion: string;
var
  Reg: TRegistry;
begin
  Result := 'Not installed';

  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKey_Local_Machine;

    // Version 1.0 = 2002
    // Version 1.1 = 2003
    // Version 2.0 = 2005

    // We pass False to OpenKey so it won't create a key that isn't found
    if Reg.OpenKey('\SOFTWARE\Microsoft\.NETFramework\policy\v1.0', False) then
    begin
      Result := 'v1.0';
      Reg.CloseKey;
    end;

    if Reg.OpenKey('\SOFTWARE\Microsoft\.NETFramework\policy\v1.1', False) then
    begin
      if Result = 'Not installed' then
        Result := 'v1.1'
      else
        // we found at least one other version
        Result := Result + ', v1.1';
      Reg.CloseKey;
    end;

    if Reg.OpenKey('\SOFTWARE\Microsoft\.NETFramework\policy\v2.0', False) then
    begin
      if Result = 'Not installed' then
        Result := 'v2.0'
      else
        // we found at least one other version
        Result := Result + ', v2.0';
      Reg.CloseKey;
    end;
  finally
    Reg.Free;
  end;
end;

procedure TAboutBox.FormCreate(Sender: TObject);
begin
  // I create this dynamically so it's not necessary to install the VCL
  LabelEffect1 := TLabelEffect.Create(self);
  with LabelEffect1 do
  begin
    Parent := Panel1;
    Caption := 'AsmIDE';
    Font.Color := clGray;
    Font.Size := 20;
    Font.Style := [fsBold];
    Top := 20;
    Left := 160;
  end;

  lblFramework.Caption := FrameworkVersion;
  lblWindows.Caption := WindowsVersion;
  lblService.Caption := ServicePackLevel;
end;

function TAboutBox.WindowsVersion: string;
var
  verInfo: TOSVersionInfo;
  Winver: string;
begin
// Get Windows Version:
  verInfo.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
  if GetVersionEx(verInfo) then
  begin
    case verInfo.dwPlatformId of
      VER_PLATFORM_WIN32s: Winver := 'Win16 running Win32s'; // we can't use this, anyway
      VER_PLATFORM_WIN32_WINDOWS: Winver := 'Win9x'; // we have to use errout for this
      VER_PLATFORM_WIN32_NT: // we prefer an NT or better platform!
        begin
          if verInfo.dwMajorVersion = 3 then
            Winver := 'WinNT, v3'
          else if verInfo.dwMajorVersion = 4 then
            Winver := 'WinNT, v4'
          else if (verInfo.dwMajorVersion = 5) and (verInfo.dwMinorVersion = 0) then
            Winver := 'Win2k'
          else if (verInfo.dwMajorVersion = 5) and (verInfo.dwMinorVersion = 1) then
            Winver := 'WinXP'
          else if (verInfo.dwMajorVersion = 5) and (verInfo.dwMinorVersion = 2) then
            Winver := 'WinServer 2003'
          else if (verInfo.dwMajorVersion = 6) {and (verInfo.dwMinorVersion = 0) } then
            Winver := 'Vista'
          else
            // it's probably newer than above - we know it's a full 32 bit platform
            Winver := IntToStr(verInfo.dwMajorVersion) + '.' + IntToStr(verInfo.dwMinorVersion);
        end;
    end;
  end
  else
    WinVer := 'Unknown'; // should never happen

  Result:=Winver;
end;

function TAboutBox.ServicePackLevel: string;
type
  _OSVERSIONINFOEXA = record
    OSVersionInfo: _OSVERSIONINFOA;
    wServicePackMajor: WORD;
    wServicePackMinor: WORD;
    wSuiteMask: WORD;
    wProductType: BYTE;
    wReserved: BYTE;
  end;
  TOSVersionInfoEx = _OSVERSIONINFOEXA;
var
  OSVerInfo: TOSVersionInfoEx;
begin
  FillChar(OSVerInfo, SizeOf(OSVerInfo), 0);
  OSVerInfo.OSVersionInfo.dwOSVersionInfoSize := SizeOf(OSVerInfo);

  { Try to use the OSVersionInfoEx structure first -- GetVersionEx will
    return False if it doesn't understand. }
  if GetVersionEx(OSVerInfo.OSVersionInfo) then
    Result := IntToStr(OSVerInfo.wServicePackMajor)
  else
    Result := 'Unknown';
end;

procedure TAboutBox.lblHomePageClick(Sender: TObject);
begin
 if (Sender is TLabel) then
    with (Sender as Tlabel) do
      ShellExecute(Application.Handle,
        PChar('open'),
        PChar(Caption),
        PChar(0),
        nil,
        SW_NORMAL);
end;

end.

